import typing as t

from model import __version__ as _version
from model.processing.data_manager import load_pipeline, load_transformed_dataset
from model.config.core import config

pipeline_file_name = f"{config.app_conf.pipeline_save_file}{_version}.pkl"
transformed_fie = f"{config.app_conf.transformed_fie}{_version}.pkl"
_recommend_pipe = load_pipeline(file_name=pipeline_file_name)
_movies = load_transformed_dataset(file_name=transformed_fie)

def recommend(movie: str) -> t.List[str]:
    index = _movies[_movies['title'] == movie].index[0]
    distances = sorted(list(enumerate(_recommend_pipe[index])), reverse=True, key=lambda x: x[1])
    recommended_movie_names = []
    for i in distances[1:6]:
        # fetch the movie poster
        movie_id = _movies.iloc[i[0]].movie_id
        recommended_movie_names.append(_movies.iloc[i[0]].title)

    return recommended_movie_names

def make_recommendation(
    *,
    input_data: dict,
) -> dict:
    """Make a recommendation using a saved model pipeline."""

    results = {"recommendations": None, "version": _version}

    # if not errors:
    recommendations = recommend(
        movie=input_data['movie']
    )
    results = {
        "recommendations": recommendations,  # type: ignore
        "version": _version,
        # "errors": errors,
    }

    return results