import numpy as np
from config.core import config
from pipeline import merge_dataset, subset_dataframe, convert_dataset, collapse_dataset, split_dataset, create_tags_variable, drop_variables, create_new_tags
from processing.data_manager import load_dataset, save_pipeline, save_dataset
from sklearn.feature_extraction.text import CountVectorizer
from sklearn.metrics.pairwise import cosine_similarity

def run_training() -> None:
    """Train the model."""

    # read training data
    movies = load_dataset(file_name=config.app_conf.training_movies_file)
    credits = load_dataset(file_name=config.app_conf.training_credits_file)

    # pipeline
    data = merge_dataset(movies, credits, config.model_conf.variable_to_merge)
    data = subset_dataframe(data, config.model_conf.variables_to_keep)
    data.dropna(inplace=True)
    data = convert_dataset(data, config.model_conf.variables_to_convert, config.model_conf.variables_top_three, config.model_conf.variables_to_fetch_director)
    data = collapse_dataset(data, config.model_conf.variables_to_collapse)
    data = split_dataset(data, config.model_conf.variables_to_split)
    data = create_tags_variable(data, config.model_conf.variable_new, config.model_conf.variables_to_create_tag)
    data = drop_variables(data, config.model_conf.variables_to_drop)
    data = create_new_tags(data, config.model_conf.variable_new)

    # persist transformed data
    save_dataset(pipeline_to_persist=data)

    # fit model
    cv = CountVectorizer(max_features=config.model_conf.max_features, stop_words=config.model_conf.stop_words)
    vector = cv.fit_transform(data[config.model_conf.variable_new]).toarray()
    similarity = cosine_similarity(vector)

    # persist trained model
    save_pipeline(pipeline_to_persist=similarity)

if __name__ == "__main__":
    run_training()